@@ -5,13 +5,15 @@ import copy
 
 from typing import Any, Dict, List, Set
 
+from flask import g
+
 from nsj_rest_lib.dao.dao_base import DAOBase
 from nsj_rest_lib.descriptor.dto_field import DTOFieldFilter
 from nsj_rest_lib.descriptor.filter_operator import FilterOperator
 from nsj_rest_lib.dto.dto_base import DTOBase
 from nsj_rest_lib.entity.entity_base import EntityBase
 from nsj_rest_lib.entity.filter import Filter
-from nsj_rest_lib.exception import DTOListFieldConfigException
+from nsj_rest_lib.exception import DTOListFieldConfigException, ConflictException, NotFoundException
 from nsj_rest_lib.injector_factory_base import NsjInjectorFactoryBase
 
 
@@ -32,6 +34,8 @@ class ServiceBase:
         self._dto_class = dto_class
         self._entity_class = entity_class
         self._dto_post_response_class = dto_post_response_class
+        self._created_by_property = 'criado_por'
+        self._updated_by_property = 'atualizado_por'
 
     def get(
         self,
@@ -325,9 +329,30 @@ class ServiceBase:
                 for entity_field, value in relation_field_map.items():
                     if hasattr(entity, entity_field):
                         setattr(entity, entity_field, value)
+            
+            # Setando campos criado_por e atualizado_por quando existirem
+            if hasattr(g, 'profile') and g.profile is not None:
+                auth_type_is_api_key = g.profile["authentication_type"] == "api_key"
+                user = g.profile["email"]
+                if insert and hasattr(entity, self._created_by_property):
+                    if not auth_type_is_api_key:
+                        setattr(entity, self._created_by_property, user)
+                    else:
+                        value = getattr(entity, self._created_by_property)
+                        if value is None or value == '':
+                            raise ValueError(f"É necessário preencher o campo '{self._created_by_property}'.")
+                if hasattr(entity, self._updated_by_property):
+                    if not auth_type_is_api_key:
+                        setattr(entity, self._updated_by_property, user)
+                    else:
+                        value = getattr(entity, self._updated_by_property)
+                        if value is None or value == '':
+                            raise ValueError(f"É necessário preencher o campo '{self._updated_by_property}'")
 
             # Invocando o DAO
             if insert:
+                if self.entity_exists(entity):
+                    raise ConflictException(f"Já existe um registro no banco com o identificador '{getattr(entity, entity_pk_field)}'")
                 entity = self._dao.insert(entity)
             else:
                 # Montando os filtros
@@ -499,3 +524,21 @@ class ServiceBase:
 
         # Chamando o DAO para a exclusão
         self._dao.delete(entity_filters)
+    
+    def entity_exists(self, entity: EntityBase):
+        # Getting values
+        entity_pk_field = entity.get_pk_field()
+        entity_pk_value = getattr(entity, entity_pk_field)
+        grupo_empresarial = getattr(entity, 'grupo_empresarial', None)
+        tenant = getattr(entity, 'tenant', None)
+
+        if entity_pk_value is None:
+            return False
+
+        # Searching entity in DB 
+        try:
+            self._dao.get(entity_pk_value, [entity.get_pk_column_name()], grupo_empresarial=grupo_empresarial, tenant=tenant)
+        except NotFoundException as e:
+            return False
+        
+        return True
